local min, max = 0, 0
local ALTERNATE_POWER_INDEX = ALTERNATE_POWER_INDEX
local currentPower = {}
local sortedPowerList = {}
local highestUnit = nil

local function getPowerPercentage(unit)
	 local powerPercentage = (currentPower[unit] / max) * 100
	 if powerPercentage > 80 then
		powerPercentage = (" |cFFFF0000%s|r"):format(powerPercentage) -- red
	 elseif powerPercentage > 50 then
		powerPercentage = (" |cFFFFFF00%s|r"):format(powerPercentage) -- yellow
	 elseif powerPercentage < 50 then
		powerPercentage = (" |cFF33FF00%s|r"):format(powerPercentage) -- green
	 end
	 return powerPercentage
end

local function sortFunc(a, b) return currentPower[a] > currentPower[b] end
local function updatePowers()
    wipe(sortedPowerList)
    for k in pairs(currentPower) do
        sortedPowerList[#sortedPowerList+1] = k
    end
    table.sort(sortedPowerList, sortFunc)
    highestUnit = sortedPowerList[1]
    updateDisplay()
end

Atramedes = CreateFrame("Frame")
Atramedes:RegisterEvent("UNIT_POWER")
Atramedes:RegisterEvent("UNIT_MAXPOWER")
Atramedes:RegisterEvent("ADDON_LOADED")
Atramedes:SetScript("OnEvent", function(self, event, unit, powerType)
	if event == "ADDON_LOADED" then
		self.db = LibStub("AceDB-3.0"):New("AtramedesDB", defaults, true)
	end
    if powerType ~= "ALTERNATE" then return end
    if not UnitInRaid(unit) then return end
    local n = UnitName(unit)
    if event == "UNIT_MAXPOWER" then
        local barType, minPower = UnitAlternatePowerInfo(unit)
        min = minPower
        max = UnitPowerMax(unit, ALTERNATE_POWER_INDEX)
    end
    currentPower[n] = UnitPower(unit, ALTERNATE_POWER_INDEX)
    updatePowers()
end)

local addon = Atramedes
local defaults = {
	profile = {
		posx = nil,
		posy = nil,
		lock = nil,
		width = 100,
		height = 80,
	}
}

local AceGUI = nil

local lock = "Interface\\AddOns\\Atramedes\\Textures\\icons\\lock"
local unlock = "Interface\\AddOns\\Atramedes\\Textures\\icons\\un_lock"

local anchor = nil

local hexColors = {}
for k, v in pairs(RAID_CLASS_COLORS) do
	hexColors[k] = ("|cff%02x%02x%02x"):format(v.r * 255, v.g * 255, v.b * 255)
end

-- Helper table to cache colored player names.
local coloredNames = setmetatable({}, {__index =
	function(self, key)
		if type(key) == "nil" then return nil end
		local _, class = UnitClass(key)
		if class then
			self[key] = hexColors[class] .. key .. "|r"
			return self[key]
		else
			return key
		end
	end
})

local L = LibStub("AceLocale-3.0"):GetLocale("Atramedes")

-------------------------------------------------------------------------------
-- Display Window
--

local function onDragStart(self) self:StartMoving() end
local function onDragStop(self)
	self:StopMovingOrSizing()
	local s = self:GetEffectiveScale()
	addon.db.profile.posx = self:GetLeft() * s
	addon.db.profile.posy = self:GetTop() * s
end
local function OnDragHandleMouseDown(self) self.frame:StartSizing("BOTTOMRIGHT") end
local function OnDragHandleMouseUp(self, button) self.frame:StopMovingOrSizing() end
local function onResize(self, width, height)
	addon.db.profile.width = width
	addon.db.profile.height = height
end

local locked = nil
local function lockDisplay()
	if locked then return end
	anchor:EnableMouse(false)
	anchor:SetMovable(false)
	anchor:SetResizable(false)
	anchor:RegisterForDrag()
	anchor:SetScript("OnSizeChanged", nil)
	anchor:SetScript("OnDragStart", nil)
	anchor:SetScript("OnDragStop", nil)
	anchor.drag:Hide()
	locked = true
end

local function unlockDisplay()
	if not locked then return end
	anchor:EnableMouse(true)
	anchor:SetMovable(true)
	anchor:SetResizable(true)
	anchor:RegisterForDrag("LeftButton")
	anchor:SetScript("OnSizeChanged", onResize)
	anchor:SetScript("OnDragStart", onDragStart)
	anchor:SetScript("OnDragStop", onDragStop)
	anchor.drag:Show()
	locked = nil
end

local function updateLockButton()
	if not anchor then return end
	anchor.lock:SetNormalTexture(addon.db.profile.lock and unlock or lock)
end

local function toggleLock()
	if addon.db.profile.lock then
		unlockDisplay()		
	else
		lockDisplay()
	end
	addon.db.profile.lock = not addon.db.profile.lock
	updateLockButton()
end

local function onControlEnter(self)
	GameTooltip:ClearLines()
	GameTooltip:SetOwner(self, "ANCHOR_TOPLEFT")
	GameTooltip:AddLine(self.tooltipHeader)
	GameTooltip:AddLine(self.tooltipText, 1, 1, 1, 1)
	GameTooltip:Show()
end
local function onControlLeave() GameTooltip:Hide() end

local function onNormalClose()
	addon:Close()
end

local function breakThings()
	anchor.lock:SetScript("OnEnter", nil)
	anchor.lock:SetScript("OnLeave", nil)
	anchor.lock:SetScript("OnClick", nil)
	anchor.close:SetScript("OnEnter", nil)
	anchor.close:SetScript("OnLeave", nil)
	anchor.close:SetScript("OnClick", nil)
end

local function makeThingsWork()
	anchor.lock:SetScript("OnEnter", onControlEnter)
	anchor.lock:SetScript("OnLeave", onControlLeave)
	anchor.lock:SetScript("OnClick", toggleLock)
	anchor.close:SetScript("OnEnter", onControlEnter)
	anchor.close:SetScript("OnLeave", onControlLeave)
	anchor.close:SetScript("OnClick", onNormalClose)
end

local function ensureDisplay()
	if anchor then return end

	local display = CreateFrame("Frame", "AtramedesAnchor", UIParent)
	display:SetWidth(addon.db.profile.width)
	display:SetHeight(addon.db.profile.height)
	display:SetMinResize(100, 30)
	display:SetClampedToScreen(true)
	local bg = display:CreateTexture(nil, "PARENT")
	bg:SetAllPoints(display)
	bg:SetBlendMode("BLEND")
	bg:SetTexture(0, 0, 0, 0.3)
	display.background = bg

	local close = CreateFrame("Button", nil, display)
	close:SetPoint("BOTTOMRIGHT", display, "TOPRIGHT", -2, 2)
	close:SetHeight(16)
	close:SetWidth(16)
	close.tooltipHeader = L["Close"]
	close.tooltipText = L["Closes the Atramedes loudness display."]
	close:SetNormalTexture("Interface\\AddOns\\Atramedes\\Textures\\icons\\close")
	display.close = close

	local lock = CreateFrame("Button", nil, display)
	lock:SetPoint("BOTTOMLEFT", display, "TOPLEFT", 2, 2)
	lock:SetHeight(16)
	lock:SetWidth(16)
	lock.tooltipHeader = L["Toggle lock"]
	lock.tooltipText = L["Toggle whether or not the Atramedes window should be locked or not."]
	display.lock = lock

	local header = display:CreateFontString(nil, "OVERLAY", "GameFontNormal")
	header:SetText("Atramedes")
	header:SetPoint("BOTTOM", display, "TOP", 0, 4)
	display.header = header

	local text = display:CreateFontString(nil, "OVERLAY", "GameFontNormal")
	text:SetFont("atramedesfont", 12)
	text:SetText("")
	text:SetAllPoints(display)
	display.text = text
	display:SetScript("OnShow", function() text:SetText("|cff777777:-P|r") end)

	local drag = CreateFrame("Frame", nil, display)
	drag.frame = display
	drag:SetFrameLevel(display:GetFrameLevel() + 10) -- place this above everything
	drag:SetWidth(16)
	drag:SetHeight(16)
	drag:SetPoint("BOTTOMRIGHT", display, -1, 1)
	drag:EnableMouse(true)
	drag:SetScript("OnMouseDown", OnDragHandleMouseDown)
	drag:SetScript("OnMouseUp", OnDragHandleMouseUp)
	drag:SetAlpha(0.5)
	display.drag = drag

	local tex = drag:CreateTexture(nil, "BACKGROUND")
	tex:SetTexture("Interface\\AddOns\\Atramedes\\Textures\\draghandle")
	tex:SetWidth(16)
	tex:SetHeight(16)
	tex:SetBlendMode("ADD")
	tex:SetPoint("CENTER", drag)

	anchor = display

	local x = addon.db.profile.posx
	local y = addon.db.profile.posy
	if x and y then
		local s = display:GetEffectiveScale()
		display:ClearAllPoints()
		display:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", x / s, y / s)
	else
		display:ClearAllPoints()
		display:SetPoint("CENTER", UIParent)
	end

	addon:RestyleWindow()
end

function addon:RestyleWindow()
	updateLockButton()
	if addon.db.profile.lock then
		locked = nil
		lockDisplay()
	else
		locked = true
		unlockDisplay()
	end
end

-------------------------------------------------------------------------------
-- Display Updater
--

local function updateDisplay()
	local tooLoud = {}
	for i = 1, #sortedPowerList do
		local n = sortedPowerList[i]
		if n and UnitExists(n) and not UnitIsDeadOrGhost(n) then
			tooLoud[#tooLoud + 1] = coloredNames[n]..getPowerPercentage(n)
		end
		if #tooLoud > 4 or i > 25 then break end
	end

	if #tooLoud == 0 then
		anchor.text:SetText("|cff777777:-P|r")
	else
		anchor.text:SetText(table.concat(tooLoud, "\n"))
		wipe(tooLoud)
	end
end

local function resetAnchor()
	anchor:ClearAllPoints()
	anchor:SetPoint("CENTER", UIParent)
	anchor:SetWidth(defaults.profile.width)
	anchor:SetHeight(defaults.profile.height)
	addon.db.profile.posx = nil
	addon.db.profile.posy = nil
	addon.db.profile.width = nil
	addon.db.profile.height = nil
end

do
	local function onControlEnter(widget, event, value)
		GameTooltip:ClearLines()
		GameTooltip:SetOwner(widget.frame, "ANCHOR_CURSOR")
		GameTooltip:AddLine(widget.text and widget.text:GetText() or widget.label:GetText())
		GameTooltip:AddLine(widget:GetUserData("tooltip"), 1, 1, 1, 1)
		GameTooltip:Show()
	end
	local function onControlLeave() GameTooltip:Hide() end
end

function addon:Close()	
	if anchor then
		anchor.header:SetText("Atramedes")
		-- configure mode, reset the background color.
		anchor.background:SetTexture(0, 0, 0, 0.3)
		anchor:Hide()
	end
end

function addon:Open()
	-- Make sure the anchor is there
	ensureDisplay()
	-- Unbreak the lock+close buttons
	makeThingsWork()
	-- Start the show!
	anchor:Show()
	updateDisplay()
end

function addon:Test()
	-- Make sure the anchor is there
	ensureDisplay()
	-- Close ourselves in case we entered configure mode DURING a boss fight.
	self:Close()
	-- Break the lock+close buttons
	breakThings()
	anchor:Show()
end

SlashCmdList.Atramedes = function(input)
	input = input:trim()
	if input == "reset" then
		resetAnchor()
	elseif input == "lock" then
		toggleLock()
	elseif input == "unlock" then
		toggleLock()
	else
		addon:Open()
	end	
end
SLASH_Atramedes1 = "/Atramedes"
